@@ -14,7 +14,6 @@ use XML::Checker::Parser;
 
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
-use pgBackRest::FileCommon;
 
 ####################################################################################################################################
 # CONSTRUCTOR
@@ -33,53 +32,44 @@ sub new
     (
         my $strOperation,
         $self->{strFileName},
-        my $bCached
     ) =
         logDebugParam
         (
             __PACKAGE__ . '->new', \@_,
             {name => 'strFileName', required => false},
-            {name => 'bCached', default => false}
         );
 
     # Load the doc from a file if one has been defined
     if (defined($self->{strFileName}))
     {
-        if ($bCached)
+        my $oParser = XML::Checker::Parser->new(ErrorContext => 2, Style => 'Tree');
+        $oParser->set_sgml_search_path(dirname(dirname($0)) . '/doc/xml/dtd');
+
+        my $oTree;
+
+        eval
         {
-            $self->oDoc = XMLin(fileStringRead($self->{strFileName}));
-        }
-        else
-        {
-            my $oParser = XML::Checker::Parser->new(ErrorContext => 2, Style => 'Tree');
-            $oParser->set_sgml_search_path(dirname(dirname($0)) . '/doc/xml/dtd');
-
-            my $oTree;
-
-            eval
+            local $XML::Checker::FAIL = sub
             {
-                local $XML::Checker::FAIL = sub
-                {
-                    my $iCode = shift;
+                my $iCode = shift;
 
-                    die XML::Checker::error_string($iCode, @_);
-                };
-
-                $oTree = $oParser->parsefile($self->{strFileName});
-
-                return true;
-            }
-            # Report any error that stopped parsing
-            or do
-            {
-                my $strException = $EVAL_ERROR;
-                $strException =~ s/at \/.*?$//s;               # remove module line number
-                die "malformed xml in '$self->{strFileName}':\n" . trim($strException);
+                die XML::Checker::error_string($iCode, @_);
             };
 
-            # Parse and build the doc
-            $self->{oDoc} = $self->build($self->parse(${$oTree}[0], ${$oTree}[1]));
+            $oTree = $oParser->parsefile($self->{strFileName});
+
+            return true;
         }
+        # Report any error that stopped parsing
+        or do
+        {
+            my $strException = $EVAL_ERROR;
+            $strException =~ s/at \/.*?$//s;               # remove module line number
+            die "malformed xml in '$self->{strFileName}':\n" . trim($strException);
+        };
+
+        # Parse and build the doc
+        $self->{oDoc} = $self->build($self->parse(${$oTree}[0], ${$oTree}[1]));
     }
     # Else create a blank doc
     else
@@ -518,13 +508,75 @@ sub nodeRemove
 
     if (!$bRemove)
     {
-        confess &log(ERROR, "child was not found in node");
+        confess &log(ERROR, "child was not found in node, could not be removed");
     }
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
 }
 
+####################################################################################################################################
+# nodeReplace
+#
+# Replace a child node with one or more child nodes.
+####################################################################################################################################
+sub nodeReplace
+{
+    my $self = shift;
+
+    # Assign function parameters, defaults, and log debug info
+    my
+    (
+        $strOperation,
+        $oChildRemove,
+        $oyChildReplace,
+    ) =
+        logDebugParam
+        (
+            __PACKAGE__ . '->nodeReplace', \@_,
+            {name => 'oChildRemove', trace => true},
+            {name => 'oChildReplace', trace => true},
+        );
+
+    my $bReplace = false;
+    my $iReplaceIdx = undef;
+    my $iReplaceTotal = undef;
+    my $oDoc = $self->{oDoc};
+
+    # Error if there are no children
+    if (!defined($$oDoc{children}))
+    {
+        confess &log(ERROR, "node has no children");
+    }
+
+    for (my $iIndex = 0; $iIndex < @{$$oDoc{children}}; $iIndex++)
+    {
+        if ($$oDoc{children}[$iIndex] == $oChildRemove->{oDoc})
+        {
+            splice(@{$$oDoc{children}}, $iIndex, 1);
+            splice(@{$$oDoc{children}}, $iIndex, 0, @{$oyChildReplace});
+
+            $iReplaceIdx = $iIndex;
+            $iReplaceTotal = scalar(@{$oyChildReplace});
+            $bReplace = true;
+            last;
+        }
+    }
+
+    if (!$bReplace)
+    {
+        confess &log(ERROR, "child was not found in node, could not be replaced");
+    }
+
+    # Return from function and log return values if any
+    return logDebugReturn
+    (
+        $strOperation,
+        {name => 'iReplaceIdx', value => $iReplaceIdx, trace => true},
+        {name => 'iReplaceTotal', value => $iReplaceTotal, trace => true},
+    );
+}
+
 ####################################################################################################################################
 # nameGet
 ####################################################################################################################################
