@@ -15,7 +15,6 @@ use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Config::Config;
 use pgBackRest::Config::ConfigHelp;
-use pgBackRest::FileCommon;
 use pgBackRest::Version;
 
 ####################################################################################################################################
@@ -95,7 +94,7 @@ sub process
 
     foreach my $strOption (sort(keys(%{$oOptionRule})))
     {
-        if ($strOption =~ /^test/ || $strOption eq 'fork')
+        if ($strOption =~ /^test/ || $strOption eq OPTION_ARCHIVE_MAX_MB)
         {
             next;
         }
@@ -410,7 +409,7 @@ sub helpDataWrite
         "1;\n";
 
     # Write the perl module into the lib path
-    fileStringWrite(dirname(dirname($0)) . '/lib/pgBackRest/Config/ConfigHelpData.pm', $strHelpData, false);
+    $oManifest->storage()->put(dirname(dirname($0)) . '/lib/pgBackRest/Config/ConfigHelpData.pm', $strHelpData);
 
     # Return from function and log return values if any
     logDebugReturn($strOperation);
@@ -859,23 +858,15 @@ sub helpCommandDocGetOptionFind
     {
         $oOption = $$oConfigHelpData{&CONFIG_HELP_OPTION}{$strOption};
 
-        if (defined($$oOption{&CONFIG_HELP_SECTION}))
+        if (defined($$oOption{&CONFIG_HELP_SECTION}) && $strSection ne $strCommand)
         {
             $strSection = $$oOption{&CONFIG_HELP_SECTION};
-
-            if ($strSection eq CONFIG_SECTION_COMMAND)
-            {
-                $strSection = CONFIG_SECTION_GENERAL;
-            }
-        }
-        else
-        {
-            $strSection = CONFIG_SECTION_GENERAL;
         }
 
         if (($strSection ne CONFIG_SECTION_GENERAL && $strSection ne CONFIG_SECTION_LOG &&
-             $strSection ne CONFIG_SECTION_STANZA && $strSection ne CONFIG_SECTION_EXPIRE) ||
-             $strSection eq $strCommand)
+             $strSection ne CONFIG_SECTION_REPOSITORY && $strSection ne CONFIG_SECTION_STANZA &&
+             $strSection ne CONFIG_SECTION_EXPIRE) ||
+            $strSection eq $strCommand)
         {
             $strSection = CONFIG_HELP_COMMAND;
         }
