@@ -7,6 +7,7 @@ use parent 'BackRestDoc::Common::DocRender';
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess);
+use English '-no_match_vars';
 
 use Exporter qw(import);
     our @EXPORT = qw();
@@ -17,7 +18,6 @@ use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Config::Config;
-use pgBackRest::FileCommon;
 use pgBackRest::Version;
 
 use pgBackRestTest::Common::ExecuteTest;
@@ -26,6 +26,11 @@ use pgBackRestTest::Common::HostGroupTest;
 
 use BackRestDoc::Common::DocManifest;
 
+####################################################################################################################################
+# User that's building the docs
+####################################################################################################################################
+use constant DOC_USER                                              => getpwuid($UID) eq 'root' ? 'ubuntu' : getpwuid($UID) . '';
+
 ####################################################################################################################################
 # CONSTRUCTOR
 ####################################################################################################################################
@@ -52,7 +57,7 @@ sub new
         );
 
     # Create the class hash
-    my $self = $class->SUPER::new($strType, $oManifest, $strRenderOutKey);
+    my $self = $class->SUPER::new($strType, $oManifest, $bExe, $strRenderOutKey);
     bless $self, $class;
 
     if (defined($self->{oSource}{hyCache}))
@@ -103,7 +108,7 @@ sub executeKey
     # Add user to command
     my $strCommand = $self->{oManifest}->variableReplace(trim($oCommand->fieldGet('exe-cmd')));
     my $strUser = $self->{oManifest}->variableReplace($oCommand->paramGet('user', false, 'postgres'));
-    $strCommand = ($strUser eq 'vagrant' ? '' : ('sudo ' . ($strUser eq 'root' ? '' : "-u $strUser "))) . $strCommand;
+    $strCommand = ($strUser eq DOC_USER ? '' : ('sudo ' . ($strUser eq 'root' ? '' : "-u $strUser "))) . $strCommand;
 
     # Format and split command
     $strCommand =~ s/[ ]*\n[ ]*/ \\\n    /smg;
@@ -233,32 +238,13 @@ sub execute
 
                     # Trim off extra linefeeds before and after
                     $strOutput =~ s/^\n+|\n$//g;
-
-                    if ($strCommand =~ / pgbackrest /)
-                    {
-                        $strOutput =~ s/^                             //smg;
-                        $strOutput =~ s/^[0-9]{4}-[0-1][0-9]-[0-3][0-9] [0-2][0-9]:[0-6][0-9]:[0-6][0-9]\.[0-9]{3} //smg;
-                    }
-
-                    # my @stryOutput = split("\n", $strOutput);
-                    # $$hCacheValue{stdout} = \@stryOutput;
                 }
 
                 if (defined($$hCacheKey{'err-expect'}) && defined($oExec->{strErrorLog}) && $oExec->{strErrorLog} ne '')
                 {
-                    # my $strError = $oExec->{strErrorLog};
-                    # $strError =~ s/^\n+|\n$//g;
-                    # my @stryError = split("\n", $strError);
-                    # $$hCacheValue{stderr} = \@stryError;
-
                     $strOutput .= $oExec->{strErrorLog};
                 }
 
-                # if (defined($$hCacheValue{stderr}))
-                # {
-                #     $strOutput .= join("\n", @{$$hCacheValue{stderr}});
-                # }
-
                 if ($$hCacheKey{output} && defined($$hCacheKey{highlight}) && $$hCacheKey{highlight}{filter} && defined($strOutput))
                 {
                     my $strHighLight = @{$$hCacheKey{highlight}{list}}[0];
@@ -331,7 +317,7 @@ sub execute
                         }
                         else
                         {
-                            $strOutput .= (defined($strOutput) ? "\n" : '') . $stryOutput[@stryOutput - 1];
+                            $strOutput .= (defined($strOutput) ? "\n" : '') . $stryOutput[-1];
                         }
                     }
                 }
@@ -572,27 +558,31 @@ sub backrestConfig
                 }
             }
 
-            my $strLocalFile = "/home/vagrant/data/pgbackrest.conf";
+            my $strLocalFile = '/home/' . DOC_USER . '/data/pgbackrest.conf';
 
             # Save the ini file
-            iniSave($strLocalFile, $self->{config}{$strHostName}{$$hCacheKey{file}}, true);
+            $self->{oManifest}->storage()->put($strLocalFile, iniRender($self->{config}{$strHostName}{$$hCacheKey{file}}, true));
 
-            $oHost->copyTo($strLocalFile, $$hCacheKey{file}, $oConfig->paramGet('owner', false, 'postgres:postgres'), '640');
+            $oHost->copyTo(
+                $strLocalFile, $$hCacheKey{file},
+                $self->{oManifest}->variableReplace($oConfig->paramGet('owner', false, 'postgres:postgres')), '640');
 
             # Remove the log-console-stderr option before pushing into the cache
             # ??? This is not very pretty and should be replaced with a general way to hide config options
             my $oConfigClean = dclone($self->{config}{$strHostName}{$$hCacheKey{file}});
             delete($$oConfigClean{&CONFIG_SECTION_GLOBAL}{&OPTION_LOG_LEVEL_STDERR});
+            delete($$oConfigClean{&CONFIG_SECTION_GLOBAL}{&OPTION_LOG_TIMESTAMP});
+            delete($$oConfigClean{&CONFIG_SECTION_GLOBAL}{&OPTION_REPO_S3_VERIFY_SSL});
 
             if (keys(%{$$oConfigClean{&CONFIG_SECTION_GLOBAL}}) == 0)
             {
                 delete($$oConfigClean{&CONFIG_SECTION_GLOBAL});
             }
 
-            iniSave("${strLocalFile}.clean", $oConfigClean, true);
+            $self->{oManifest}->storage()->put("${strLocalFile}.clean", iniRender($oConfigClean, true));
 
             # Push config file into the cache
-            $strConfig = fileStringRead("${strLocalFile}.clean");
+            $strConfig = ${$self->{oManifest}->storage()->get("${strLocalFile}.clean")};
 
             my @stryConfig = undef;
 
@@ -667,12 +657,13 @@ sub postgresConfig
                 confess &log(ERROR, "cannot configure postgres on host ${strHostName} because the host does not exist");
             }
 
-            my $strLocalFile = '/home/vagrant/data/postgresql.conf';
+            my $strLocalFile = '/home/' . DOC_USER . '/data/postgresql.conf';
             $oHost->copyFrom($$hCacheKey{file}, $strLocalFile);
 
             if (!defined(${$self->{'pg-config'}}{$strHostName}{$$hCacheKey{file}}{base}) && $self->{bExe})
             {
-                ${$self->{'pg-config'}}{$strHostName}{$$hCacheKey{file}}{base} = fileStringRead($strLocalFile);
+                ${$self->{'pg-config'}}{$strHostName}{$$hCacheKey{file}}{base} =
+                    ${$self->{oManifest}->storage()->get($strLocalFile)};
             }
 
             my $oConfigHash = $self->{'pg-config'}{$strHostName}{$$hCacheKey{file}};
@@ -722,7 +713,7 @@ sub postgresConfig
             # Save the conf file
             if ($self->{bExe})
             {
-                fileStringWrite($strLocalFile, $$oConfigHash{base} .
+                $self->{oManifest}->storage()->put($strLocalFile, $$oConfigHash{base} .
                                 (defined($strConfig) ? "\n# pgBackRest Configuration\n${strConfig}\n" : ''));
 
                 $oHost->copyTo($strLocalFile, $$hCacheKey{file}, 'postgres:postgres', '640');
@@ -778,20 +769,19 @@ sub hostKey
     my $hCacheKey =
     {
         name => $self->{oManifest}->variableReplace($oHost->paramGet('name')),
-        user => $self->{oManifest}->variableReplace($oHost->paramGet('user')),
         image => $self->{oManifest}->variableReplace($oHost->paramGet('image')),
     };
 
+    if (defined($oHost->paramGet('option', false)))
+    {
+        $$hCacheKey{option} = $self->{oManifest}->variableReplace($oHost->paramGet('option'));
+    }
+
     if (defined($oHost->paramGet('os', false)))
     {
         $$hCacheKey{os} = $self->{oManifest}->variableReplace($oHost->paramGet('os'));
     }
 
-    if (defined($oHost->paramGet('mount', false)))
-    {
-        $$hCacheKey{mount} = $self->{oManifest}->variableReplace($oHost->paramGet('mount'));
-    }
-
     # Return from function and log return values if any
     return logDebugReturn
     (
@@ -975,23 +965,26 @@ sub sectionChildProcess
                 executeTest("mkdir -p ~/data/$$hCacheKey{name}/etc");
 
                 my $oHost = new pgBackRestTest::Common::HostTest(
-                    $$hCacheKey{name}, "doc-$$hCacheKey{name}", $$hCacheKey{image}, $$hCacheKey{user}, $$hCacheKey{os},
-                    defined($$hCacheKey{mount}) ? [$$hCacheKey{mount}] : undef);
+                    $$hCacheKey{name}, "doc-$$hCacheKey{name}", $$hCacheKey{image},
+                    $self->{oManifest}->variableReplace($oChild->paramGet('user')), $$hCacheKey{os},
+                    defined($oChild->paramGet('mount', false)) ?
+                        [$self->{oManifest}->variableReplace($oChild->paramGet('mount'))] : undef,
+                    $$hCacheKey{option});
 
                 $self->{host}{$$hCacheKey{name}} = $oHost;
                 $self->{oManifest}->variableSet("host-$$hCacheKey{name}-ip", $oHost->{strIP}, true);
                 $$hCacheValue{ip} = $oHost->{strIP};
 
-                # Execute cleanup commands
+                # Add to the host group
+                my $oHostGroup = hostGroupGet();
+                $oHostGroup->hostAdd($oHost);
+
+                # Execute initialize commands
                 foreach my $oExecute ($oChild->nodeList('execute', false))
                 {
                     $self->execute($oSection, $$hCacheKey{name}, $oExecute, $iDepth + 1, false);
                 }
 
-                # Add to the host group
-                my $oHostGroup = hostGroupGet();
-                $oHostGroup->hostAdd($oHost);
-
                 $self->cachePush($strCacheType, $hCacheKey, $hCacheValue);
             }
         }
