@@ -14,7 +14,6 @@ use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Config::Config;
 use pgBackRest::Config::ConfigHelp;
-use pgBackRest::FileCommon;
 
 use BackRestDoc::Common::DocRender;
 
@@ -308,17 +307,18 @@ sub docGet
             if ($iStableReleaseTotal == 0)
             {
                 $oSection = $oDoc->nodeAdd('section', undef, {id => 'supported'});
-                $oSection->nodeAdd('title')->textSet("Supported Stable Releases");
+                $oSection->nodeAdd('title')->textSet("Stable Releases");
             }
 
             $iStableReleaseTotal++;
+            $bTOC = false;
         }
         else
         {
             if ($iUnsupportedReleaseTotal == 0)
             {
                 $oSection = $oDoc->nodeAdd('section', undef, {id => 'unsupported'});
-                $oSection->nodeAdd('title')->textSet("Unsupported Releases");
+                $oSection->nodeAdd('title')->textSet("Pre-Stable Releases");
             }
 
             $iUnsupportedReleaseTotal++;
