@@ -126,7 +126,7 @@ sub process
     {
         my $oMenuBody = $oHtmlBuilder->bodyGet()->addNew(HTML_DIV, 'page-menu')->addNew(HTML_DIV, 'menu-body');
 
-        if ($self->{strRenderOutKey} ne 'index')
+        if ($self->{strRenderOutKey} ne 'index' && defined($self->{oManifest}->renderOutGet(RENDER_TYPE_HTML, 'index', true)))
         {
             my $oRenderOut = $self->{oManifest}->renderOutGet(RENDER_TYPE_HTML, 'index');
 
@@ -219,7 +219,10 @@ sub sectionProcess
             {name => 'iDepth'}
         );
 
-    &log($iDepth == 1 ? INFO : DEBUG, ('    ' x ($iDepth + 1)) . 'process section: ' . $oSection->paramGet('id'));
+    if ($oSection->paramGet('log'))
+    {
+        &log(INFO, ('    ' x ($iDepth + 1)) . 'process section: ' . $oSection->paramGet('path'));
+    }
 
     if ($iDepth > 3)
     {
@@ -275,7 +278,6 @@ sub sectionProcess
     my $oSectionBodyElement = $oSectionElement->addNew(HTML_DIV, "section-body");
 
     # Process each child
-    my $oSectionBodyExe;
     my $iSectionNo = 1;
 
     foreach my $oChild ($oSection->nodeList())
