@@ -15,7 +15,6 @@ use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Config::ConfigHelp;
-use pgBackRest::FileCommon;
 
 use BackRestDoc::Common::DocManifest;
 
@@ -111,7 +110,10 @@ sub sectionProcess
             {name => 'iDepth'}
         );
 
-    &log($iDepth == 1 ? INFO : DEBUG, ('    ' x ($iDepth + 1)) . 'process section: ' . $oSection->paramGet('id'));
+    if ($oSection->paramGet('log'))
+    {
+        &log(INFO, ('    ' x ($iDepth + 1)) . 'process section: ' . $oSection->paramGet('path'));
+    }
 
     # Create the section
     my $strSectionTitle = $self->processText($oSection->nodeGet('title')->textGet());
