@@ -35,7 +35,8 @@ use pgBackRest::Common::Exception;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::String;
 use pgBackRest::Config::Config;
-use pgBackRest::FileCommon;
+use pgBackRest::Storage::Local;
+use pgBackRest::Storage::Posix::Driver;
 use pgBackRest::Version;
 
 use pgBackRestTest::Common::ExecuteTest;
@@ -123,6 +124,9 @@ eval
     my $strDocHtml = "${strDocPath}/output/html";
     my $strDocExe = "${strDocPath}/doc.pl";
 
+    my $oStorageDoc = new pgBackRest::Storage::Local(
+        $strDocPath, new pgBackRest::Storage::Posix::Driver({bFileSync => false, bPathSync => false}));
+
     # Determine if this is a dev release
     my $bDev = BACKREST_VERSION =~ /dev$/;
     my $strVersion = $bDev ? 'dev' : BACKREST_VERSION;
@@ -130,21 +134,21 @@ eval
     if ($bBuild)
     {
         # Remove permanent cache file
-        fileRemove("${strDocPath}/resource/exe.cache", true);
+        $oStorageDoc->remove("${strDocPath}/resource/exe.cache", {bIgnoreMissing => true});
 
         # Remove all docker containers to get consistent IP address assignments
-        executeTest('docker rm -f $(docker ps -a -q)');
+        executeTest('docker rm -f $(docker ps -a -q)', {bSuppressError => true});
 
         # Generate deployment docs for RHEL/Centos 6
         &log(INFO, "Generate RHEL/CentOS 6 documentation");
 
-        executeTest("${strDocExe} --deploy --keyword=co6 --out=pdf");
+        executeTest("${strDocExe} --deploy --keyword=co6 --out=pdf", {bShowOutputAsync => true});
         executeTest("${strDocExe} --deploy --cache-only --keyword=co6 --out=pdf --var=\"project-name=Crunchy BackRest\"");
 
         # Generate deployment docs for Debian
         &log(INFO, "Generate Debian/Ubuntu documentation");
 
-        executeTest("${strDocExe} --deploy");
+        executeTest("${strDocExe} --deploy", {bShowOutputAsync => true});
         executeTest("${strDocExe} --deploy --cache-only --out=man --out=html --var=project-url-root=index.html");
     }
 
