@@ -15,9 +15,9 @@ use Exporter qw(import);
 ####################################################################################################################################
 # Exception codes
 ####################################################################################################################################
-use constant ERROR_MINIMUM                                          => 100;
+use constant ERROR_MINIMUM                                          => 25;
     push @EXPORT, qw(ERROR_MINIMUM);
-use constant ERROR_MAXIMUM                                          => 199;
+use constant ERROR_MAXIMUM                                          => 125;
     push @EXPORT, qw(ERROR_MAXIMUM);
 
 use constant ERROR_ASSERT                                           => ERROR_MINIMUM;
@@ -50,8 +50,8 @@ use constant ERROR_POSTMASTER_RUNNING                               => ERROR_MIN
     push @EXPORT, qw(ERROR_POSTMASTER_RUNNING);
 use constant ERROR_PROTOCOL                                         => ERROR_MINIMUM + 14;
     push @EXPORT, qw(ERROR_PROTOCOL);
-use constant ERROR_RESTORE_PATH_NOT_EMPTY                           => ERROR_MINIMUM + 15;
-    push @EXPORT, qw(ERROR_RESTORE_PATH_NOT_EMPTY);
+use constant ERROR_PATH_NOT_EMPTY                                   => ERROR_MINIMUM + 15;
+    push @EXPORT, qw(ERROR_PATH_NOT_EMPTY);
 use constant ERROR_FILE_OPEN                                        => ERROR_MINIMUM + 16;
     push @EXPORT, qw(ERROR_FILE_OPEN);
 use constant ERROR_FILE_READ                                        => ERROR_MINIMUM + 17;
@@ -144,12 +144,18 @@ use constant ERROR_LINK_OPEN                                        => ERROR_MIN
     push @EXPORT, qw(ERROR_LINK_OPEN);
 use constant ERROR_ARCHIVE_DISABLED                                 => ERROR_MINIMUM + 62;
     push @EXPORT, qw(ERROR_ARCHIVE_DISABLED);
-use constant ERROR_ARCHIVE_DIR_INVALID                              => ERROR_MINIMUM + 63;
-    push @EXPORT, qw(ERROR_ARCHIVE_DIR_INVALID);
-use constant ERROR_BACKUP_DIR_INVALID                               => ERROR_MINIMUM + 64;
-    push @EXPORT, qw(ERROR_BACKUP_DIR_INVALID);
+use constant ERROR_FILE_OWNER                                       => ERROR_MINIMUM + 63;
+    push @EXPORT, qw(ERROR_FILE_OWNER);
+use constant ERROR_USER_MISSING                                     => ERROR_MINIMUM + 64;
+    push @EXPORT, qw(ERROR_USER_MISSING);
 use constant ERROR_OPTION_COMMAND                                   => ERROR_MINIMUM + 65;
     push @EXPORT, qw(ERROR_OPTION_COMMAND);
+use constant ERROR_GROUP_MISSING                                    => ERROR_MINIMUM + 66;
+    push @EXPORT, qw(ERROR_GROUP_MISSING);
+use constant ERROR_PATH_EXISTS                                      => ERROR_MINIMUM + 67;
+    push @EXPORT, qw(ERROR_PATH_EXISTS);
+use constant ERROR_FILE_EXISTS                                      => ERROR_MINIMUM + 68;
+    push @EXPORT, qw(ERROR_FILE_EXISTS);
 
 use constant ERROR_INVALID_VALUE                                    => ERROR_MAXIMUM - 2;
     push @EXPORT, qw(ERROR_INVALID_VALUE);
@@ -253,4 +259,32 @@ sub isException
 
 push @EXPORT, qw(isException);
 
+####################################################################################################################################
+# exceptionCode
+#
+# Extract the error code from an exception - if a Perl exception return ERROR_UNKNOWN.
+####################################################################################################################################
+sub exceptionCode
+{
+    my $oException = shift;
+
+    return isException($oException) ? $oException->code() : ERROR_UNKNOWN;
+}
+
+push @EXPORT, qw(exceptionCode);
+
+####################################################################################################################################
+# exceptionMessage
+#
+# Extract the error message from an exception - if a Perl exception return bare exception.
+####################################################################################################################################
+sub exceptionMessage
+{
+    my $oException = shift;
+
+    return isException($oException) ? $oException->message() : $oException;
+}
+
+push @EXPORT, qw(exceptionMessage);
+
 1;
