@@ -6,6 +6,7 @@ package pgBackRest::Common::Log;
 use strict;
 use warnings FATAL => qw(all);
 use Carp qw(confess longmess);
+use English '-no_match_vars';
 
 use Exporter qw(import);
     our @EXPORT = qw();
@@ -71,6 +72,14 @@ my $strLogFileCache = undef;
 my $strLogLevelFile = OFF;
 my $strLogLevelConsole = OFF;
 my $strLogLevelStdErr = WARN;
+my $bLogTimestamp = true;
+
+# Flags to limit banner printing until there is actual output
+my $bLogFileExists;
+my $bLogFileFirst;
+
+# Allow log to be globally enabled or disabled with logEnable() and logDisable()
+my $bLogDisable = 0;
 
 # Test globals
 my $bTest = false;
@@ -110,28 +119,25 @@ sub logFileSet
     # Only open the log file if file logging is enabled
     if ($strLogLevelFile ne OFF)
     {
-        filePathCreate(dirname($strFile), '0770', true, true);
+        # Load Storage::Helper module
+        require pgBackRest::Storage::Helper;
+        pgBackRest::Storage::Helper->import();
+
+        storageLocal()->pathCreate(dirname($strFile), {strMode => '0770', bIgnoreExists => true, bCreateParent => true});
 
         $strFile .= '.log';
-        my $bExists = false;
+        $bLogFileExists = -e $strFile ? true : false;
+        $bLogFileFirst = true;
 
-        if (-e $strFile)
+        if (!sysopen($hLogFile, $strFile, O_WRONLY | O_CREAT | O_APPEND, oct('0660')))
         {
-            $bExists = true;
+            logErrorResult(ERROR_FILE_OPEN, "unable to open log file '${strFile}'", $OS_ERROR);
         }
 
-        $hLogFile = fileOpen($strFile, O_WRONLY | O_CREAT | O_APPEND, '0660');
-
-        if ($bExists)
-        {
-            syswrite($hLogFile, "\n");
-        }
-
-        syswrite($hLogFile, "-------------------PROCESS START-------------------\n");
-
         # Write out anything that was cached before the file was opened
         if (defined($strLogFileCache))
         {
+            logBanner();
             syswrite($hLogFile, $strLogFileCache);
             undef($strLogFileCache);
         }
@@ -140,6 +146,26 @@ sub logFileSet
 
 push @EXPORT, qw(logFileSet);
 
+####################################################################################################################################
+# logBanner
+#
+# Output a banner on the first log entry written to a file
+####################################################################################################################################
+sub logBanner
+{
+    if ($bLogFileFirst)
+    {
+        if ($bLogFileExists)
+        {
+            syswrite($hLogFile, "\n");
+        }
+
+        syswrite($hLogFile, "-------------------PROCESS START-------------------\n");
+    }
+
+    $bLogFileFirst = false;
+}
+
 ####################################################################################################################################
 # logLevelSet - set the log level for file and console
 ####################################################################################################################################
@@ -148,10 +174,7 @@ sub logLevelSet
     my $strLevelFileParam = shift;
     my $strLevelConsoleParam = shift;
     my $strLevelStdErrParam = shift;
-
-    # Load FileCommon module
-    require pgBackRest::FileCommon;
-    pgBackRest::FileCommon->import();
+    my $bLogTimestampParam = shift;
 
     if (defined($strLevelFileParam))
     {
@@ -182,10 +205,35 @@ sub logLevelSet
 
         $strLogLevelStdErr = uc($strLevelStdErrParam);
     }
+
+    if (defined($bLogTimestampParam))
+    {
+        $bLogTimestamp = $bLogTimestampParam;
+    }
 }
 
 push @EXPORT, qw(logLevelSet);
 
+####################################################################################################################################
+# logDisable
+####################################################################################################################################
+sub logDisable
+{
+    $bLogDisable++;
+}
+
+push @EXPORT, qw(logDisable);
+
+####################################################################################################################################
+# logEnable
+####################################################################################################################################
+sub logEnable
+{
+    $bLogDisable--;
+}
+
+push @EXPORT, qw(logEnable);
+
 ####################################################################################################################################
 # logDebugParam
 #
@@ -258,6 +306,7 @@ sub logDebugProcess
 
     # Process each parameter hash
     my $oParam = shift;
+    my $bOptionalBlock = false;
 
     # Strip the package name off strFunction if it's pgBackRest
     $strFunction =~ s/^pgBackRest[^\:]*\:\://;
@@ -265,14 +314,43 @@ sub logDebugProcess
     while (defined($oParam))
     {
         my $strParamName = $$oParam{name};
+        my $bParamOptional = defined($oParam->{optional}) && $oParam->{optional};
+        my $bParamRequired = !defined($oParam->{required}) || $oParam->{required};
         my $oValue;
 
+        # If param is optional then the optional block has been entered
+        if ($bParamOptional)
+        {
+            if (defined($oParam->{required}))
+            {
+                confess &log(ASSERT, "cannot define 'required' for optional parameter '${strParamName}'");
+            }
+
+            $bParamRequired = false;
+            $bOptionalBlock = true;
+        }
+
+        # Don't allow non-optional parameters once optional block has started
+        if ($bParamOptional != $bOptionalBlock)
+        {
+            confess &log(ASSERT, "non-optional parameter '${strParamName}' invalid after optional parameters");
+        }
+
         # Push the return value into the return value array
         if ($strType eq DEBUG_PARAM)
         {
-            if (defined($$oyParamRef[$iIndex]))
+            if ($bParamOptional)
             {
-                push(@oyResult, $$oyParamRef[$iIndex]);
+                $oValue = $$oyParamRef[$iIndex]->{$strParamName};
+            }
+            else
+            {
+                $oValue = $$oyParamRef[$iIndex];
+            }
+
+            if (defined($oValue))
+            {
+                push(@oyResult, $oValue);
             }
             else
             {
@@ -280,9 +358,9 @@ sub logDebugProcess
                 $$oParamHash{$strParamName}{default} = true;
             }
 
-            $oValue = $oyResult[@oyResult - 1];
+            $oValue = $oyResult[-1];
 
-            if (!defined($oValue) && (!defined($${oParam}{required}) || $${oParam}{required}))
+            if (!defined($oValue) && $bParamRequired)
             {
                 confess &log(ASSERT, "${strParamName} is required in ${strFunction}");
             }
@@ -310,7 +388,6 @@ sub logDebugProcess
 
         if (!defined($$oParam{log}) || $$oParam{log})
         {
-
             # If the parameter is a hash but not blessed then represent it as a string
             # ??? This should go away once the inputs to logDebug can be changed
             if (ref($oValue) eq 'HASH' && !blessed($oValue))
@@ -333,7 +410,11 @@ sub logDebugProcess
 
         # Get the next parameter hash
         $oParam = shift;
-        $iIndex++;
+
+        if (!$bParamOptional)
+        {
+            $iIndex++;
+        }
     }
 
     if (defined($strDetail) && $iIndex == 0)
@@ -406,6 +487,8 @@ sub logDebugBuild
     return $rResult;
 }
 
+push @EXPORT, qw(logDebugBuild);
+
 ####################################################################################################################################
 # logDebugOut
 ####################################################################################################################################
@@ -476,6 +559,20 @@ sub logException
 
 push @EXPORT, qw(logException);
 
+####################################################################################################################################
+# logErrorResult
+####################################################################################################################################
+sub logErrorResult
+{
+    my $iCode = shift;
+    my $strMessage = shift;
+    my $strResult = shift;
+
+    confess &log(ERROR, $strMessage . (defined($strResult) ? ": $strResult" : ''), $iCode);
+}
+
+push @EXPORT, qw(logErrorResult);
+
 ####################################################################################################################################
 # LOG - log messages
 ####################################################################################################################################
@@ -539,7 +636,7 @@ sub log
         $iCode = ERROR_UNKNOWN;
     }
 
-    $strMessageFormat = (defined($iCode) ? "[${iCode}]: " : '') . $strMessageFormat;
+    $strMessageFormat = (defined($iCode) ? sprintf('[%03d]: ', $iCode) : '') . $strMessageFormat;
 
     # Indent subsequent lines of the message if it has more than one line - makes the log more readable
     if (defined($iIndent))
@@ -549,9 +646,13 @@ sub log
     }
     else
     {
-        $strMessageFormat =~ s/\n/\n                                    /g;
+        # Indent subsequent message lines so they align
+        $bLogTimestamp ?
+            $strMessageFormat =~ s/\n/\n                                        /g :
+            $strMessageFormat =~ s/\n/\n            /g
     }
 
+    # Indent TRACE and debug levels so they are distinct from normal messages
     if ($strLevel eq TRACE || $strLevel eq TEST)
     {
         $strMessageFormat =~ s/\n/\n        /g;
@@ -562,82 +663,92 @@ sub log
         $strMessageFormat =~ s/\n/\n    /g;
         $strMessageFormat = '    ' . $strMessageFormat;
     }
-    elsif ($strLevel eq ERROR || $strLevel eq ASSERT)
-    {
-        $strMessageFormat =~ s/\n/\n       /g;
-    }
 
     # Format the message text
     my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
 
     $strMessageFormat =
-        timestampFormat() . sprintf('.%03d P%02d', (gettimeofday() - int(gettimeofday())) * 1000,
-        (defined($iProcessId) ? $iProcessId : 0)) .
+        ($bLogTimestamp ? timestampFormat() . sprintf('.%03d ', (gettimeofday() - int(gettimeofday())) * 1000) : '') .
+        sprintf('P%02d', defined($iProcessId) ? $iProcessId : 0) .
         (' ' x (7 - length($strLevel))) . "${strLevel}: ${strMessageFormat}\n";
 
-    # Output to stderr depending on log level
-    if (!$rExtra->{bLogConsole} && $iLogLevelRank <= $oLogLevelRank{$strLogLevelStdErr}{rank})
+    # Skip output if disabled
+    if (!$bLogDisable)
     {
-        if ($strLogLevelStdErr ne PROTOCOL)
+        # Output to stderr depending on log level
+        if (!$rExtra->{bLogConsole} && $iLogLevelRank <= $oLogLevelRank{$strLogLevelStdErr}{rank})
         {
-            syswrite(*STDERR, $strLevel . (defined($iCode) ? " [${iCode}]" : '') . ': ');
+            if ($strLogLevelStdErr ne PROTOCOL)
+            {
+                syswrite(*STDERR, $strLevel . (defined($iCode) ? sprintf(' [%03d]: ', $iCode) : '') . ': ');
+            }
+
+            syswrite(*STDERR, "${strMessage}\n");
+            $rExtra->{bLogConsole} = true;
         }
-
-        syswrite(*STDERR, "${strMessage}\n");
-        $rExtra->{bLogConsole} = true;
-    }
-    # Else output to stdout depending on log level and test flag
-    elsif (!$rExtra->{bLogConsole} && $iLogLevelRank <= $oLogLevelRank{$strLogLevelConsole}{rank} || $bTest && $strLevel eq TEST)
-    {
-        if (!$bSuppressLog)
-        {
-            syswrite(*STDOUT, $strMessageFormat);
-        }
-
-        # If in test mode and this is a test messsage then delay so the calling process has time to read the message
-        if ($bTest && $strLevel eq TEST && $fTestDelay > 0)
-        {
-            usleep($fTestDelay * 1000000);
-        }
-
-        $rExtra->{bLogConsole} = true;
-    }
-
-    # Output to file depending on log level and test flag
-    if (!$rExtra->{bLogLogFile} && $iLogLevelRank <= $oLogLevelRank{$strLogLevelFile}{rank})
-    {
-        if (defined($hLogFile) || (defined($strLogLevelFile) && $strLogLevelFile ne OFF))
+        # Else output to stdout depending on log level and test flag
+        elsif (!$rExtra->{bLogConsole} && $iLogLevelRank <= $oLogLevelRank{$strLogLevelConsole}{rank} ||
+               $bTest && $strLevel eq TEST)
         {
             if (!$bSuppressLog)
             {
-                if (defined($hLogFile))
-                {
-                    syswrite($hLogFile, $strMessageFormat);
-                }
-                else
-                {
-                    $strLogFileCache .= $strMessageFormat;
-                }
+                syswrite(*STDOUT, $strMessageFormat);
 
-                if ($strLevel eq ASSERT ||
-                    ($strLevel eq ERROR && ($strLogLevelFile eq DEBUG || $strLogLevelFile eq TRACE)))
-                {
-                    my $strStackTrace = longmess() . "\n";
-                    $strStackTrace =~ s/\n/\n                                   /g;
+                # This is here for debugging purposes - it's not clear how best to make it into a switch
+                # if ($strLevel eq ASSERT || $strLevel eq ERROR)
+                # {
+                #     my $strStackTrace = longmess() . "\n";
+                #     $strStackTrace =~ s/\n/\n                                   /g;
+                #     syswrite(*STDOUT, $strStackTrace);
+                # }
+            }
 
+            # If in test mode and this is a test messsage then delay so the calling process has time to read the message
+            if ($bTest && $strLevel eq TEST && $fTestDelay > 0)
+            {
+                usleep($fTestDelay * 1000000);
+            }
+
+            $rExtra->{bLogConsole} = true;
+        }
+
+        # Output to file depending on log level and test flag
+        if (!$rExtra->{bLogLogFile} && $iLogLevelRank <= $oLogLevelRank{$strLogLevelFile}{rank})
+        {
+            if (defined($hLogFile) || (defined($strLogLevelFile) && $strLogLevelFile ne OFF))
+            {
+                if (!$bSuppressLog)
+                {
                     if (defined($hLogFile))
                     {
-                        syswrite($hLogFile, $strStackTrace);
+                        logBanner();
+                        syswrite($hLogFile, $strMessageFormat);
                     }
                     else
                     {
-                        $strLogFileCache .= $strStackTrace;
+                        $strLogFileCache .= $strMessageFormat;
+                    }
+
+                    if ($strLevel eq ASSERT ||
+                        ($strLevel eq ERROR && ($strLogLevelFile eq DEBUG || $strLogLevelFile eq TRACE)))
+                    {
+                        my $strStackTrace = longmess() . "\n";
+                        $strStackTrace =~ s/\n/\n                                   /g;
+
+                        if (defined($hLogFile))
+                        {
+                            syswrite($hLogFile, $strStackTrace);
+                        }
+                        else
+                        {
+                            $strLogFileCache .= $strStackTrace;
+                        }
                     }
                 }
             }
-        }
 
-        $rExtra->{bLogFile} = true;
+            $rExtra->{bLogFile} = true;
+        }
     }
 
     # Throw a typed exception if code is defined
@@ -696,4 +807,34 @@ sub testCheck
 
 push @EXPORT, qw(testCheck);
 
+####################################################################################################################################
+# logLevel - get the current log levels
+####################################################################################################################################
+sub logLevel
+{
+    return ($strLogLevelFile, $strLogLevelConsole, $strLogLevelStdErr, $bLogTimestamp);
+}
+
+push @EXPORT, qw(logLevel);
+
+####################################################################################################################################
+# logFileCacheClear - Clear the log file cache
+####################################################################################################################################
+sub logFileCacheClear
+{
+    undef($strLogFileCache);
+}
+
+push @EXPORT, qw(logFileCacheClear);
+
+####################################################################################################################################
+# logFileCache - Get the log file cache
+####################################################################################################################################
+sub logFileCache
+{
+    return $strLogFileCache;
+}
+
+push @EXPORT, qw(logFileCache);
+
 1;
