@@ -32,6 +32,24 @@ sub trim
 
 push @EXPORT, qw(trim);
 
+####################################################################################################################################
+# coalesce - return first defined parameter
+####################################################################################################################################
+sub coalesce
+{
+    foreach my $strParam (@_)
+    {
+        if (defined($strParam))
+        {
+            return $strParam;
+        }
+    }
+
+    return;
+}
+
+push @EXPORT, qw(coalesce);
+
 ####################################################################################################################################
 # commonPrefix
 #
