@@ -11,16 +11,19 @@ use Exporter qw(import);
     our @EXPORT = qw();
 use File::Basename qw(dirname);
 
+use pgBackRest::Backup::Common;
+use pgBackRest::Backup::Info;
 use pgBackRest::Common::Log;
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::String;
-use pgBackRest::BackupCommon;
-use pgBackRest::BackupInfo;
+use pgBackRest::Backup::Common;
+use pgBackRest::Backup::Info;
 use pgBackRest::Config::Config;
-use pgBackRest::File;
+use pgBackRest::InfoCommon;
 use pgBackRest::Manifest;
-use pgBackRest::Protocol::Common;
-use pgBackRest::Protocol::Protocol;
+use pgBackRest::Protocol::Helper;
+use pgBackRest::Protocol::Storage::Helper;
+use pgBackRest::Storage::Helper;
 
 ####################################################################################################################################
 # Info constants
@@ -48,17 +51,19 @@ use constant INFO_STANZA_STATUS_NO_BACKUP_MESSAGE                   => 'no valid
 use constant INFO_KEY_CODE                                          => 'code';
 use constant INFO_KEY_DELTA                                         => 'delta';
 use constant INFO_KEY_FORMAT                                        => 'format';
-use constant INFO_KEY_ID                                            => 'id';
+use constant INFO_KEY_ID                                            => INFO_HISTORY_ID;
 use constant INFO_KEY_LABEL                                         => 'label';
+use constant INFO_KEY_MAX                                           => 'max';
+use constant INFO_KEY_MIN                                           => 'min';
 use constant INFO_KEY_MESSAGE                                       => 'message';
 use constant INFO_KEY_PRIOR                                         => 'prior';
 use constant INFO_KEY_REFERENCE                                     => 'reference';
 use constant INFO_KEY_SIZE                                          => 'size';
 use constant INFO_KEY_START                                         => 'start';
 use constant INFO_KEY_STOP                                          => 'stop';
-use constant INFO_KEY_SYSTEM_ID                                     => 'system-id';
+use constant INFO_KEY_SYSTEM_ID                                     => INFO_SYSTEM_ID;
 use constant INFO_KEY_TYPE                                          => 'type';
-use constant INFO_KEY_VERSION                                       => 'version';
+use constant INFO_KEY_VERSION                                       => INFO_DB_VERSION;
 
 ####################################################################################################################################
 # CONSTRUCTOR
@@ -95,65 +100,12 @@ sub process
     # Get stanza if specified
     my $strStanza = optionTest(OPTION_STANZA) ? optionGet(OPTION_STANZA) : undef;
 
-    # Create the file object
-    my $oFile = new pgBackRest::File
-    (
-        $strStanza,
-        optionGet(OPTION_REPO_PATH),
-        protocolGet(BACKUP)
-    );
-
     # Get the stanza list with all info
-    my $oyStanzaList = $self->stanzaList($oFile, $strStanza);
+    my $oyStanzaList = $self->stanzaList($strStanza);
 
     if (optionTest(OPTION_OUTPUT, INFO_OUTPUT_TEXT))
     {
-        my $strOutput;
-
-        foreach my $oStanzaInfo (@{$oyStanzaList})
-        {
-            # Add an LF between stanzas
-            $strOutput .= defined($strOutput) ? "\n" : '';
-
-            # Output stanza name and status
-            $strOutput .=
-                'stanza: ' . $$oStanzaInfo{&INFO_STANZA_NAME} . "\n" .
-                '    status: ' . ($$oStanzaInfo{&INFO_SECTION_STATUS}{&INFO_KEY_CODE} == 0 ? INFO_STANZA_STATUS_OK :
-                INFO_STANZA_STATUS_ERROR . ' (' . $$oStanzaInfo{&INFO_SECTION_STATUS}{&INFO_KEY_MESSAGE} . ')') . "\n";
-
-            # Output information for each stanza backup, from oldest to newest
-            foreach my $oBackupInfo (@{$$oStanzaInfo{&INFO_BACKUP_SECTION_BACKUP}})
-            {
-                $strOutput .=
-                    "\n".
-                    '    ' . $$oBackupInfo{&INFO_KEY_TYPE} . ' backup: ' . $$oBackupInfo{&INFO_KEY_LABEL} . "\n" .
-
-                    '        start / stop timestamp: ' .
-                    timestampFormat(undef, $$oBackupInfo{&INFO_SECTION_TIMESTAMP}{&INFO_KEY_START}) .
-                    ' / ' .
-                    timestampFormat(undef, $$oBackupInfo{&INFO_SECTION_TIMESTAMP}{&INFO_KEY_STOP}) . "\n" .
-
-                    '        database size: ' .
-                    (defined($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_KEY_SIZE}) ?
-                        fileSizeFormat($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_KEY_SIZE}) : '') .
-                    ', backup size: ' .
-                    (defined($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_KEY_DELTA}) ?
-                        fileSizeFormat($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_KEY_DELTA}) : '') . "\n" .
-
-                    '        repository size: ' .
-                    (defined($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_SECTION_REPO}{&INFO_KEY_SIZE}) ?
-                        fileSizeFormat($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_SECTION_REPO}{&INFO_KEY_SIZE}) : '') .
-                    ', repository backup size: ' .
-                    (defined($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_SECTION_REPO}{&INFO_KEY_DELTA}) ?
-                        fileSizeFormat($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_SECTION_REPO}{&INFO_KEY_DELTA}) : '') . "\n";
-
-                # List the backup reference chain, if any, for this backup
-                if (defined($$oBackupInfo{&INFO_KEY_REFERENCE}))
-                {
-                    $strOutput .= '        backup reference list: ' . (join(', ', @{$$oBackupInfo{&INFO_KEY_REFERENCE}})) . "\n";
-                }
-            }
-        }
+        my $strOutput = $self->formatText($oyStanzaList);
 
         if (defined($strOutput))
         {
@@ -161,22 +113,13 @@ sub process
         }
         else
         {
-            syswrite(*STDOUT, 'No stanzas exist in ' . $oFile->pathGet(PATH_BACKUP) . ".\n");
+            syswrite(*STDOUT, 'No stanzas exist in ' . storageRepo()->pathGet() . ".\n");
         }
     }
     elsif (optionTest(OPTION_OUTPUT, INFO_OUTPUT_JSON))
     {
         my $oJSON = JSON::PP->new()->canonical()->pretty()->indent_length(4);
-        my $strJSON = $oJSON->encode($oyStanzaList);
-
-        syswrite(*STDOUT, $strJSON);
-
-        # On some systems a linefeed will be appended by encode() but others will not have it.  In our case there should always
-        # be a terminating linefeed.
-        if ($strJSON !~ /\n$/)
-        {
-            syswrite(*STDOUT, "\n");
-        }
+        $self->outputJSON($oJSON->encode($oyStanzaList));
     }
     else
     {
@@ -191,6 +134,218 @@ sub process
     );
 }
 
+####################################################################################################################################
+# outputJSON
+###################################################################################################################################
+sub outputJSON
+{
+    my $self = shift;
+
+    # Assign function parameters, defaults, and log debug info
+    my
+    (
+        $strOperation,
+        $strJSON,
+    ) =
+        logDebugParam
+        (
+            __PACKAGE__ . '->outputJSON', \@_,
+            {name => 'strJSON'},
+        );
+
+    syswrite(*STDOUT, $strJSON);
+
+    # On some systems a linefeed will be appended by encode() but others will not have it.  In our case there should always
+    # be a terminating linefeed.
+    if ($strJSON !~ /\n$/)
+    {
+        syswrite(*STDOUT, "\n");
+    }
+}
+
+####################################################################################################################################
+# formatText
+#
+# Format --output=text info.
+####################################################################################################################################
+sub formatText
+{
+    my $self = shift;
+
+    # Assign function parameters, defaults, and log debug info
+    my
+    (
+        $strOperation,
+        $oyStanzaList,
+    ) =
+        logDebugParam
+        (
+            __PACKAGE__ . '->formatText', \@_,
+            {name => 'oyStanzaList', trace => true},
+        );
+
+    my $strOutput;
+
+    foreach my $oStanzaInfo (@{$oyStanzaList})
+    {
+        # Output stanza name and status
+        $strOutput .= (defined($strOutput) ? "\n" : '')  . $self->formatTextStanza($oStanzaInfo) . "\n";
+
+        # Initialize the flag to indicate the first DB in the INFO_BACKUP_SECTION_DB is the current database
+        my $bDbCurrent = true;
+
+        # Loop through the DB history array for the stanza from newest to oldest
+        foreach my $hDbInfo (reverse @{$oStanzaInfo->{&INFO_BACKUP_SECTION_DB}})
+        {
+            $strOutput .= $bDbCurrent ? "\n    db (current)" : "\n    db (prior)";
+            $bDbCurrent = false;
+
+            # Get the archive information for the DB
+            foreach my $hDbArchive (@{$oStanzaInfo->{&INFO_SECTION_ARCHIVE}})
+            {
+                if ($hDbArchive->{&INFO_SECTION_DB}{&INFO_HISTORY_ID} == $hDbInfo->{&INFO_HISTORY_ID})
+                {
+                    # Output archive start / stop values
+                    $strOutput .= "\n        wal archive min/max (" . $hDbArchive->{&INFO_KEY_ID} . "): ";
+
+                    if (defined($hDbArchive->{&INFO_KEY_MIN}))
+                    {
+                        $strOutput .= $hDbArchive->{&INFO_KEY_MIN} . ' / ' . $hDbArchive->{&INFO_KEY_MAX};
+                    }
+                    else
+                    {
+                        $strOutput .= 'none present';
+                    }
+
+                    $strOutput .= "\n";
+                }
+            }
+
+            # Get information for each stanza backup for the DB, from oldest to newest
+            foreach my $oBackupInfo (@{$$oStanzaInfo{&INFO_BACKUP_SECTION_BACKUP}})
+            {
+                if ($oBackupInfo->{&INFO_SECTION_DB}{&INFO_KEY_ID} == $hDbInfo->{&INFO_HISTORY_ID})
+                {
+                    $strOutput .= "\n" . $self->formatTextBackup($oBackupInfo) . "\n";
+                }
+            }
+        }
+    }
+
+    # Return from function and log return values if any
+    return logDebugReturn
+    (
+        $strOperation,
+        {name => 'strOutput', value => $strOutput, trace => true}
+    );
+}
+
+####################################################################################################################################
+# formatTextStanza
+#
+# Format --output=text stanza info.
+####################################################################################################################################
+sub formatTextStanza
+{
+    my $self = shift;
+
+    # Assign function parameters, defaults, and log debug info
+    my
+    (
+        $strOperation,
+        $oStanzaInfo,
+    ) =
+        logDebugParam
+        (
+            __PACKAGE__ . '->formatTextStanza', \@_,
+            {name => 'oStanzaInfo', trace => true},
+        );
+
+    # Output stanza name and status
+    my $strOutput =
+        'stanza: ' . $oStanzaInfo->{&INFO_STANZA_NAME} . "\n" .
+        "    status: " . ($oStanzaInfo->{&INFO_SECTION_STATUS}{&INFO_KEY_CODE} == 0 ? INFO_STANZA_STATUS_OK :
+        INFO_STANZA_STATUS_ERROR . ' (' . $oStanzaInfo->{&INFO_SECTION_STATUS}{&INFO_KEY_MESSAGE} . ')');
+
+    # Return from function and log return values if any
+    return logDebugReturn
+    (
+        $strOperation,
+        {name => 'strOutput', value => $strOutput, trace => true}
+    );
+}
+
+####################################################################################################################################
+# formatTextBackup
+#
+# Format --output=text backup info.
+####################################################################################################################################
+sub formatTextBackup
+{
+    my $self = shift;
+
+    # Assign function parameters, defaults, and log debug info
+    my
+    (
+        $strOperation,
+        $oBackupInfo,
+    ) =
+        logDebugParam
+        (
+            __PACKAGE__ . '->formatTextBackup', \@_,
+            {name => 'oBackupInfo', trace => true},
+        );
+
+    my $strOutput =
+        '        ' . $$oBackupInfo{&INFO_KEY_TYPE} . ' backup: ' . $$oBackupInfo{&INFO_KEY_LABEL} . "\n" .
+
+        '            timestamp start/stop: ' .
+        timestampFormat(undef, $$oBackupInfo{&INFO_SECTION_TIMESTAMP}{&INFO_KEY_START}) .
+        ' / ' .
+        timestampFormat(undef, $$oBackupInfo{&INFO_SECTION_TIMESTAMP}{&INFO_KEY_STOP}) . "\n" .
+
+        "            wal start/stop: ";
+
+    if (defined($oBackupInfo->{&INFO_SECTION_ARCHIVE}{&INFO_KEY_START}) &&
+        defined($oBackupInfo->{&INFO_SECTION_ARCHIVE}{&INFO_KEY_STOP}))
+    {
+        $strOutput .=
+            $oBackupInfo->{&INFO_SECTION_ARCHIVE}{&INFO_KEY_START} . ' / ' . $oBackupInfo->{&INFO_SECTION_ARCHIVE}{&INFO_KEY_STOP};
+    }
+    else
+    {
+        $strOutput .= 'n/a';
+    }
+
+    $strOutput .=
+        "\n            database size: " .
+        (defined($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_KEY_SIZE}) ?
+            fileSizeFormat($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_KEY_SIZE}) : '') .
+        ', backup size: ' .
+        (defined($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_KEY_DELTA}) ?
+            fileSizeFormat($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_KEY_DELTA}) : '') . "\n" .
+
+        '            repository size: ' .
+        (defined($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_SECTION_REPO}{&INFO_KEY_SIZE}) ?
+            fileSizeFormat($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_SECTION_REPO}{&INFO_KEY_SIZE}) : '') .
+        ', repository backup size: ' .
+        (defined($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_SECTION_REPO}{&INFO_KEY_DELTA}) ?
+            fileSizeFormat($$oBackupInfo{&INFO_SECTION_INFO}{&INFO_SECTION_REPO}{&INFO_KEY_DELTA}) : '');
+
+    # List the backup reference chain, if any, for this backup
+    if (defined($oBackupInfo->{&INFO_KEY_REFERENCE}) && @{$oBackupInfo->{&INFO_KEY_REFERENCE}} > 0)
+    {
+        $strOutput .= "\n            backup reference list: " . (join(', ', @{$$oBackupInfo{&INFO_KEY_REFERENCE}}));
+    }
+
+    # Return from function and log return values if any
+    return logDebugReturn
+    (
+        $strOperation,
+        {name => 'strOutput', value => $strOutput, trace => true}
+    );
+}
+
 ####################################################################################################################################
 # stanzaList
 ####################################################################################################################################
@@ -202,27 +357,25 @@ sub stanzaList
     my
     (
         $strOperation,
-        $oFile,
         $strStanza
     ) =
         logDebugParam
         (
             __PACKAGE__ . '->stanzaList', \@_,
-            {name => 'oFile'},
             {name => 'strStanza', required => false}
         );
 
     my @oyStanzaList;
 
     # Run remotely
-    if ($oFile->isRemote(PATH_BACKUP))
+    if (!isRepoLocal())
     {
-        @oyStanzaList = @{$oFile->{oProtocol}->cmdExecute(OP_INFO_STANZA_LIST, [$strStanza], true)};
+        @oyStanzaList = @{protocolGet(BACKUP)->cmdExecute(OP_INFO_STANZA_LIST, [$strStanza], true)};
     }
     # Run locally
     else
     {
-        my @stryStanza = $oFile->list(PATH_BACKUP, CMD_BACKUP, undef, undef, true);
+        my @stryStanza = storageRepo()->list(CMD_BACKUP, {bIgnoreMissing => true});
 
         foreach my $strStanzaFound (@stryStanza)
         {
@@ -234,8 +387,9 @@ sub stanzaList
             my $oStanzaInfo = {};
             $$oStanzaInfo{&INFO_STANZA_NAME} = $strStanzaFound;
             ($$oStanzaInfo{&INFO_BACKUP_SECTION_BACKUP}, $$oStanzaInfo{&INFO_BACKUP_SECTION_DB}) =
-                $self->backupList($oFile, $strStanzaFound);
+                $self->backupList($strStanzaFound);
 
+            # If there are no backups then set status to no backup
             if (@{$$oStanzaInfo{&INFO_BACKUP_SECTION_BACKUP}} == 0)
             {
                 $$oStanzaInfo{&INFO_SECTION_STATUS} =
@@ -244,6 +398,7 @@ sub stanzaList
                     &INFO_KEY_MESSAGE => INFO_STANZA_STATUS_NO_BACKUP_MESSAGE
                 };
             }
+            # Else status is OK
             else
             {
                 $$oStanzaInfo{&INFO_SECTION_STATUS} =
@@ -253,6 +408,40 @@ sub stanzaList
                 };
             }
 
+            # Array to store tne min/max archive for each database for which there are archives
+            my @oyDbArchiveList = ();
+
+            # Get the current DB info (always last element in the array)
+            my $hDbCurrent = @{$oStanzaInfo->{&INFO_BACKUP_SECTION_DB}}[-1];
+            my $strDbCurrentVersion = $hDbCurrent->{&INFO_KEY_VERSION};
+            my $ullDbCurrentSystemId = $hDbCurrent->{&INFO_KEY_SYSTEM_ID};
+
+            # Loop through the DB history from oldest to newest
+            foreach my $hDbInfo (@{$oStanzaInfo->{&INFO_BACKUP_SECTION_DB}})
+            {
+                my $strArchiveStanzaPath = "archive/" . $strStanzaFound;
+                my $strDbVersion = $hDbInfo->{&INFO_KEY_VERSION};
+                my $ullDbSysId = $hDbInfo->{&INFO_KEY_SYSTEM_ID};
+
+                # With multiple DB versions, the backup.info history-id may not be the same as archive.info history-id, so the
+                # archive path must be built by retrieving the archive id given the db version and system id of the backup
+                my $oArchiveInfo = new pgBackRest::Archive::Info(storageRepo()->pathGet($strArchiveStanzaPath));
+                my $strArchiveId = $oArchiveInfo->archiveId({strDbVersion => $hDbInfo->{&INFO_KEY_VERSION},
+                    ullDbSysId => $hDbInfo->{&INFO_KEY_SYSTEM_ID}});
+                my $strArchivePath = "archive/${strStanzaFound}/${strArchiveId}";
+
+                # Fill in the archive info if available
+                my $hDbArchive = $self->dbArchiveSection($hDbInfo, $strArchiveId, $strArchivePath, $strDbCurrentVersion,
+                    $ullDbCurrentSystemId);
+                if (defined($hDbArchive))
+                {
+                    push(@oyDbArchiveList, $hDbArchive);
+                }
+            }
+
+            # Store the archive min/max for each database in the archive section
+            $oStanzaInfo->{&INFO_SECTION_ARCHIVE} = \@oyDbArchiveList;
+
             push @oyStanzaList, $oStanzaInfo;
         }
 
@@ -294,18 +483,16 @@ sub backupList
     my
     (
         $strOperation,
-        $oFile,
         $strStanza
     ) =
         logDebugParam
         (
             __PACKAGE__ . '->backupList', \@_,
-            {name => 'oFile'},
             {name => 'strStanza'}
         );
 
-    # Load or build backup.info
-    my $oBackupInfo = new pgBackRest::BackupInfo($oFile->pathGet(PATH_BACKUP, CMD_BACKUP . "/${strStanza}"));
+    # Load the backup.info but do not attempt to validate it or confirm it's existence
+    my $oBackupInfo = new pgBackRest::Backup::Info(storageRepo()->pathGet(CMD_BACKUP . "/${strStanza}"), false, false, {bIgnoreMissing => true});
 
     # Build the db list
     my @oyDbList;
@@ -314,10 +501,10 @@ sub backupList
     {
         my $oDbHash =
         {
-            &INFO_KEY_ID => $iHistoryId,
-            &INFO_KEY_VERSION =>
+            &INFO_HISTORY_ID => $iHistoryId,
+            &INFO_DB_VERSION =>
                 $oBackupInfo->get(INFO_BACKUP_SECTION_DB_HISTORY, $iHistoryId, INFO_BACKUP_KEY_DB_VERSION),
-            &INFO_KEY_SYSTEM_ID =>
+            &INFO_SYSTEM_ID =>
                 $oBackupInfo->get(INFO_BACKUP_SECTION_DB_HISTORY, $iHistoryId, INFO_BACKUP_KEY_SYSTEM_ID)
         };
 
@@ -396,4 +583,92 @@ sub backupList
     );
 }
 
+####################################################################################################################################
+# dbArchiveSection - fills the archive section for a db. Note this is a function in order to aid unit test coverage.
+####################################################################################################################################
+sub dbArchiveSection
+{
+    my $self = shift;
+
+    # Assign function parameters, defaults, and log debug info
+    my
+    (
+        $strOperation,
+        $hDbInfo,
+        $strArchiveId,
+        $strArchivePath,
+        $strDbCurrentVersion,
+        $ullDbCurrentSystemId,
+    ) =
+        logDebugParam
+        (
+            __PACKAGE__ . '->dbArchiveSection', \@_,
+            {name => 'hDbInfo'},
+            {name => 'strArchiveId'},
+            {name => 'strArchivePath'},
+            {name => 'strDbCurrentVersion'},
+            {name => 'ullDbCurrentSystemId'},
+        );
+
+    my $hDbArchive = undef;
+    my $strArchiveStart = undef;
+    my $strArchiveStop = undef;
+
+    if (storageRepo()->pathExists($strArchivePath))
+    {
+        my @stryWalMajor = storageRepo()->list($strArchivePath, {strExpression => '^[0-F]{16}$'});
+
+        # Get first WAL segment
+        foreach my $strWalMajor (@stryWalMajor)
+        {
+            my @stryWalFile = storageRepo()->list(
+                "${strArchivePath}/${strWalMajor}",
+                {strExpression => "^[0-F]{24}-[0-f]{40}(\\." . COMPRESS_EXT . "){0,1}\$"});
+
+            if (@stryWalFile > 0)
+            {
+                $strArchiveStart = substr($stryWalFile[0], 0, 24);
+                last;
+            }
+        }
+
+        # Get last WAL segment
+        foreach my $strWalMajor (sort({$b cmp $a} @stryWalMajor))
+        {
+            my @stryWalFile = storageRepo()->list(
+                "${strArchivePath}/${strWalMajor}",
+                {strExpression => "^[0-F]{24}-[0-f]{40}(\\." . COMPRESS_EXT . "){0,1}\$", strSortOrder => 'reverse'});
+
+            if (@stryWalFile > 0)
+            {
+                $strArchiveStop = substr($stryWalFile[0], 0, 24);
+                last;
+            }
+        }
+    }
+
+    # If there is an archive or the database is the current database then store it
+    if (($strDbCurrentVersion eq $hDbInfo->{&INFO_KEY_VERSION} &&
+        $ullDbCurrentSystemId == $hDbInfo->{&INFO_KEY_SYSTEM_ID}) ||
+        defined($strArchiveStart) )
+    {
+        $hDbArchive =
+        {
+            &INFO_KEY_ID => $strArchiveId,
+            &INFO_KEY_MIN => $strArchiveStart,
+            &INFO_KEY_MAX => $strArchiveStop,
+            &INFO_SECTION_DB =>
+            {
+                &INFO_HISTORY_ID => $hDbInfo->{&INFO_HISTORY_ID},
+            },
+        };
+    }
+    # Return from function and log return values if any
+    return logDebugReturn
+    (
+        $strOperation,
+        {name => 'hDbArchive', value => $hDbArchive, trace => true},
+    );
+}
+
 1;
