@@ -18,9 +18,11 @@ use pgBackRest::DbVersion;
 use pgBackRest::Common::Exception;
 use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
-use pgBackRest::File;
-use pgBackRest::FileCommon;
+use pgBackRest::Common::Wait;
 use pgBackRest::Config::Config;
+use pgBackRest::Protocol::Helper;
+use pgBackRest::Protocol::Storage::Helper;
+use pgBackRest::Storage::Helper;
 
 ####################################################################################################################################
 # File/path constants
@@ -29,6 +31,8 @@ use constant PATH_BACKUP_HISTORY                                    => 'backup.h
     push @EXPORT, qw(PATH_BACKUP_HISTORY);
 use constant FILE_MANIFEST                                          => 'backup.manifest';
     push @EXPORT, qw(FILE_MANIFEST);
+use constant FILE_MANIFEST_COPY                                     => FILE_MANIFEST . INI_COPY_EXT;
+    push @EXPORT, qw(FILE_MANIFEST_COPY);
 
 ####################################################################################################################################
 # Default match factor
@@ -162,6 +166,8 @@ use constant MANIFEST_SUBKEY_USER                                   => 'user';
 ####################################################################################################################################
 # Database locations for important files/paths
 ####################################################################################################################################
+use constant DB_PATH_PGXLOG_ARCHIVESTATUS                           => 'pg_xlog/archive_status';
+    push @EXPORT, qw(DB_PATH_PGXLOG_ARCHIVESTATUS);
 use constant DB_PATH_BASE                                           => 'base';
     push @EXPORT, qw(DB_PATH_BASE);
 use constant DB_PATH_PGCLOG                                         => 'pg_clog';
@@ -220,6 +226,8 @@ use constant DB_FILE_PREFIX_TMP                                     => 'pgsql_tm
 ####################################################################################################################################
 # Manifest locations for important files/paths
 ####################################################################################################################################
+use constant MANIFEST_PATH_PGXLOG_ARCHIVESTATUS                     => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGXLOG_ARCHIVESTATUS;
+    push @EXPORT, qw(MANIFEST_PATH_PGXLOG_ARCHIVESTATUS);
 use constant MANIFEST_PATH_BASE                                     => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_BASE;
     push @EXPORT, qw(MANIFEST_PATH_BASE);
 use constant MANIFEST_PATH_PGCLOG                                   => MANIFEST_TARGET_PGDATA . '/' . DB_PATH_PGCLOG;
@@ -291,21 +299,23 @@ sub new
     my
     (
         $strOperation,
-        $strFileName,                               # Manifest filename
-        $bLoad                                      # Load the manifest?
+        $strFileName,
+        $bLoad,
+        $oStorage,
     ) =
         logDebugParam
         (
             __PACKAGE__ . '->new', \@_,
             {name => 'strFileName', trace => true},
-            {name => 'bLoad', required => false, trace => true}
+            {name => 'bLoad', required => false, trace => true},
+            {name => 'oStorage', optional => true, default => storageRepo(), trace => true},
         );
 
     # Set defaults
     $bLoad = defined($bLoad) ? $bLoad : true;
 
     # Init object and store variables
-    my $self = $class->SUPER::new($strFileName, $bLoad);
+    my $self = $class->SUPER::new($strFileName, {bLoad => $bLoad, oStorage => $oStorage});
 
     # Return from function and log return values if any
     return logDebugReturn
@@ -554,7 +564,7 @@ sub build
     my
     (
         $strOperation,
-        $oFile,
+        $oStorageDbMaster,
         $strDbVersion,
         $strPath,
         $oLastManifest,
@@ -569,7 +579,7 @@ sub build
         logDebugParam
         (
             __PACKAGE__ . '->build', \@_,
-            {name => 'oFile'},
+            {name => 'oStorageDbMaster'},
             {name => 'strDbVersion'},
             {name => 'strPath'},
             {name => 'oLastManifest', required => false},
@@ -589,7 +599,7 @@ sub build
         # If not online then build the tablespace map from pg_tblspc path
         if (!$bOnline && !defined($hTablespaceMap))
         {
-            my $hTablespaceManifest = $oFile->manifest(PATH_DB_ABSOLUTE, $strPath . '/' . DB_PATH_PGTBLSPC);
+            my $hTablespaceManifest = $oStorageDbMaster->manifest($strPath . '/' . DB_PATH_PGTBLSPC);
             $hTablespaceMap = {};
 
             foreach my $strOid (sort(CORE::keys(%{$hTablespaceManifest})))
@@ -632,11 +642,11 @@ sub build
             confess &log(ASSERT, "cannot get manifest for '${strPath}' when no parent path is specified");
         }
 
-        $strPath = pathAbsolute($strParentPath, $strPath);
+        $strPath = $oStorageDbMaster->pathAbsolute($strParentPath, $strPath);
     }
 
     # Get the manifest for this level
-    my $hManifest = $oFile->manifest(PATH_DB_ABSOLUTE, $strPath);
+    my $hManifest = $oStorageDbMaster->manifest($strPath);
     my $strManifestType = MANIFEST_VALUE_LINK;
 
     # Loop though all paths/files/links in the manifest
@@ -678,7 +688,8 @@ sub build
 
         # Skip pg_xlog/* when doing an online backup.  WAL will be restored from the archive or stored in pg_xlog at the end of the
         # backup if the archive-copy option is set.
-        next if ($strFile =~ ('^' . MANIFEST_PATH_PGXLOG . '\/') && $bOnline);
+        next if ($bOnline && $strFile =~ ('^' . MANIFEST_PATH_PGXLOG . '\/') &&
+                 $strFile !~ ('^' . MANIFEST_PATH_PGXLOG_ARCHIVESTATUS . '$'));
 
         # Skip all directories and files that start with pgsql_tmp.  The files are removed when the server is restarted and the
         # directories are recreated.  Since temp files cannnot be created on the replica it makes sense to delete the directories
@@ -740,7 +751,7 @@ sub build
             # Check for tablespaces in PGDATA
             if (index($hManifest->{$strName}{link_destination}, "${strPath}/") == 0 ||
                 (index($hManifest->{$strName}{link_destination}, '/') != 0 &&
-                 index(pathAbsolute($strPath . '/' . DB_PATH_PGTBLSPC,
+                 index($oStorageDbMaster->pathAbsolute($strPath . '/' . DB_PATH_PGTBLSPC,
                        $hManifest->{$strName}{link_destination}) . '/', "${strPath}/") == 0))
             {
                 confess &log(ERROR, 'tablespace symlink ' . $hManifest->{$strName}{link_destination} .
@@ -794,7 +805,12 @@ sub build
 
             if ($bTablespace)
             {
-                $strFilter = $self->tablespacePathGet();
+                # Only versions >= 9.0  have the special top-level tablespace path.  Below 9.0 the database files are stored
+                # directly in the path referenced by the symlink.
+                if ($strDbVersion >= PG_VERSION_90)
+                {
+                    $strFilter = $self->tablespacePathGet();
+                }
 
                 $self->set(MANIFEST_SECTION_TARGET_PATH, MANIFEST_TARGET_PGTBLSPC, undef,
                            $self->get(MANIFEST_SECTION_TARGET_PATH, MANIFEST_TARGET_PGDATA));
@@ -805,7 +821,7 @@ sub build
 
             $strPath = dirname("${strPath}/${strName}");
 
-            $self->build($oFile, $strDbVersion, $strLinkDestination, undef, $bOnline, $hTablespaceMap, $hDatabaseMap,
+            $self->build($oStorageDbMaster, $strDbVersion, $strLinkDestination, undef, $bOnline, $hTablespaceMap, $hDatabaseMap,
                          $strFile, $bTablespace, $strPath, $strFilter, $strLinkDestination);
         }
     }
@@ -820,7 +836,9 @@ sub build
         # lead to an invalid diff/incr backup later when using timestamps to determine which files have changed.  Offline backups do
         # not wait because it makes testing much faster and Postgres should not be running (if it is the backup will not be
         # consistent anyway and the one-second resolution problem is the least of our worries).
-        my $lTimeBegin = $oFile->wait(PATH_DB_ABSOLUTE, $bOnline);
+        my $lTimeBegin =
+            $oStorageDbMaster->can('protocol') ?
+                $oStorageDbMaster->protocol()->cmdExecute(OP_WAIT, [$bOnline]) : waitRemainder($bOnline);
 
         # Check that links are valid
         $self->linkCheck();
@@ -963,8 +981,8 @@ sub linkCheck
                 {
                     my $strChildPath = $self->get(MANIFEST_SECTION_BACKUP_TARGET, $strTargetChild, MANIFEST_SUBKEY_PATH);
 
-                    if (index(
-                        pathAbsolute($strBasePath, $strChildPath) . '/', pathAbsolute($strBasePath, $strParentPath) . '/') == 0)
+                    if (index(storageLocal()->pathAbsolute(
+                        $strBasePath, $strChildPath) . '/', storageLocal()->pathAbsolute($strBasePath, $strParentPath) . '/') == 0)
                     {
                         confess &log(ERROR, 'link ' . $self->dbPathGet($strBasePath, $strTargetChild) .
                                             " (${strChildPath}) references a subdirectory of or" .
@@ -1163,12 +1181,11 @@ sub isChecksumPage
 {
     my $strFile = shift;
 
-    if (($strFile =~ ('^' . MANIFEST_TARGET_PGDATA . '\/' . DB_PATH_BASE) &&
+    if (($strFile =~ ('^' . MANIFEST_TARGET_PGDATA . '\/' . DB_PATH_BASE . '|^' . MANIFEST_TARGET_PGTBLSPC . '\/') &&
             $strFile !~ ('(' . DB_FILE_PGFILENODEMAP . '|' . DB_FILE_PGINTERNALINIT . '|' . DB_FILE_PGVERSION . ')$')) ||
         ($strFile =~ ('^' . MANIFEST_TARGET_PGDATA . '\/' . DB_PATH_GLOBAL) &&
             $strFile !~ ('(' . DB_FILE_PGFILENODEMAP . '|' . DB_FILE_PGINTERNALINIT . '|' . DB_FILE_PGVERSION . '|' .
-            DB_FILE_PGCONTROL . ')$')) ||
-        ($strFile =~ ('^' . MANIFEST_TARGET_PGTBLSPC . '\/') && $strFile !~ (DB_FILE_PGVERSION . '$')))
+            DB_FILE_PGCONTROL . ')$')))
     {
         return true;
     }
