@@ -11,18 +11,18 @@ use Cwd qw(abs_path);
 use File::Basename qw(basename dirname);
 use File::stat qw(lstat);
 
-use pgBackRest::BackupInfo;
+use pgBackRest::Backup::Info;
 use pgBackRest::Common::Exception;
+use pgBackRest::Common::Ini;
 use pgBackRest::Common::Log;
 use pgBackRest::Config::Config;
 use pgBackRest::DbVersion;
-use pgBackRest::File;
-use pgBackRest::FileCommon;
 use pgBackRest::Manifest;
 use pgBackRest::RestoreFile;
-use pgBackRest::Protocol::Common;
-use pgBackRest::Protocol::LocalProcess;
-use pgBackRest::Protocol::Protocol;
+use pgBackRest::Protocol::Helper;
+use pgBackRest::Protocol::Local::Process;
+use pgBackRest::Protocol::Storage::Helper;
+use pgBackRest::Storage::Helper;
 use pgBackRest::Version;
 
 ####################################################################################################################################
@@ -40,15 +40,7 @@ sub new
     my ($strOperation) = logDebugParam(__PACKAGE__ . '->new');
 
     # Initialize protocol
-    $self->{oProtocol} = protocolGet(BACKUP);
-
-    # Initialize default file object
-    $self->{oFile} = new pgBackRest::File
-    (
-        optionGet(OPTION_STANZA),
-        optionGet(OPTION_REPO_PATH),
-        $self->{oProtocol}
-    );
+    $self->{oProtocol} = !isRepoLocal() ? protocolGet(BACKUP) : undef;
 
     # Initialize variables
     $self->{strDbClusterPath} = optionGet(OPTION_DB_PATH);
@@ -62,22 +54,6 @@ sub new
     );
 }
 
-####################################################################################################################################
-# DESTROY
-####################################################################################################################################
-sub DESTROY
-{
-    my $self = shift;
-
-    # Assign function parameters, defaults, and log debug info
-    my ($strOperation) = logDebugParam(__PACKAGE__ . '->DESTROY');
-
-    undef($self->{oFile});
-
-    # Return from function and log return values if any
-    return logDebugReturn($strOperation);
-}
-
 ####################################################################################################################################
 # manifestOwnershipCheck
 #
@@ -225,18 +201,19 @@ sub manifestLoad
     my ($strOperation) = logDebugParam (__PACKAGE__ . '->manifestLoad');
 
     # Error if the backup set does not exist
-    if (!$self->{oFile}->exists(PATH_BACKUP_CLUSTER, $self->{strBackupSet}))
+    if (!storageRepo()->exists(STORAGE_REPO_BACKUP . "/$self->{strBackupSet}/" . FILE_MANIFEST))
     {
-        confess &log(ERROR, 'backup ' . $self->{strBackupSet} . ' does not exist');
+        confess &log(ERROR, "backup '$self->{strBackupSet}' does not exist");
     }
 
     # Copy the backup manifest to the db cluster path
-    $self->{oFile}->copy(PATH_BACKUP_CLUSTER, "$self->{strBackupSet}/" . FILE_MANIFEST,
-                         PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . FILE_MANIFEST);
+    storageDb()->copy(
+        storageRepo()->openRead(STORAGE_REPO_BACKUP . "/$self->{strBackupSet}/" . FILE_MANIFEST, {bProtocolCompress => true}),
+        $self->{strDbClusterPath} . '/' . FILE_MANIFEST);
 
     # Load the manifest into a hash
-    my $oManifest = new pgBackRest::Manifest($self->{oFile}->pathGet(PATH_DB_ABSOLUTE,
-                                                                   $self->{strDbClusterPath} . '/' . FILE_MANIFEST));
+    my $oManifest = new pgBackRest::Manifest(
+        storageDb()->pathGet($self->{strDbClusterPath} . '/' . FILE_MANIFEST), undef, {oStorage => storageDb()});
 
     # If backup is latest then set it equal to backup label, else verify that requested backup and label match
     my $strBackupLabel = $oManifest->get(MANIFEST_SECTION_BACKUP, MANIFEST_KEY_LABEL);
@@ -310,6 +287,13 @@ sub manifestLoad
         }
     }
 
+    # Issue a warning message when we remap tablespaces in postgre < 9.2
+    if ($oManifest->get(MANIFEST_SECTION_BACKUP_DB, MANIFEST_KEY_DB_VERSION) < PG_VERSION_92 &&
+       (optionTest(OPTION_TABLESPACE_MAP) || optionTest(OPTION_TABLESPACE_MAP_ALL)))
+    {
+        &log(WARN, "update pg_tablespace.spclocation with new tablespace location in PostgreSQL < " . PG_VERSION_92);
+    }
+
     # Alter manifest for remapped tablespaces
     if (defined($oTablespaceRemap))
     {
@@ -478,6 +462,9 @@ sub clean
             {name => 'oManifest'}
         );
 
+    # Db storage
+    my $oStorageDb = storageDb();
+
     # Track if files/links/paths where removed
     my %oRemoveHash =
     (
@@ -507,7 +494,7 @@ sub clean
                 $strBasePath .= "/${strTargetPath}";
             }
 
-            ${$self->{oTargetPath}}{$strTarget} = pathAbsolute($strBasePath, $strCheckPath);
+            ${$self->{oTargetPath}}{$strTarget} = $oStorageDb->pathAbsolute($strBasePath, $strCheckPath);
 
             $strCheckPath = ${$self->{oTargetPath}}{$strTarget};
 
@@ -520,7 +507,7 @@ sub clean
         &log(DETAIL, "check ${strCheckPath} exists");
 
         # Check if the path exists
-        if (!$self->{oFile}->exists(PATH_DB_ABSOLUTE, $strCheckPath))
+        if (!$oStorageDb->pathExists($strCheckPath))
         {
             confess &log(ERROR, "cannot restore to missing path ${strCheckPath}", ERROR_PATH_MISSING);
         }
@@ -533,13 +520,13 @@ sub clean
                                $oManifest->get(MANIFEST_SECTION_BACKUP_TARGET, $strTarget, MANIFEST_SUBKEY_FILE);
 
             # Check if the file exists
-            if ($self->{oFile}->exists(PATH_DB_ABSOLUTE, $strCheckFile))
+            if ($oStorageDb->exists($strCheckFile))
             {
                 # If the file exists and this is not a delta then error
                 if (!$bDelta)
                 {
                     confess &log(ERROR, "cannot restore file '${strCheckFile}' that already exists - " .
-                                        'try using --delta if this is what you intended', ERROR_RESTORE_PATH_NOT_EMPTY);
+                                        'try using --delta if this is what you intended', ERROR_PATH_NOT_EMPTY);
                 }
 
                 # Mark that a file was found
@@ -556,14 +543,14 @@ sub clean
                 ${$self->{oTargetPath}}{$strTarget} = "${$self->{oTargetPath}}{$strTarget}/" . $oManifest->tablespacePathGet();
 
                 # If this path does not exist then skip the rest of the checking - the path will be created later
-                if (!$self->{oFile}->exists(PATH_DB_ABSOLUTE, ${$self->{oTargetPath}}{$strTarget}))
+                if (!$oStorageDb->pathExists(${$self->{oTargetPath}}{$strTarget}))
                 {
                     next;
                 }
             }
 
             # Build a manifest of the path to check for existing files
-            my $hTargetManifest = $self->{oFile}->manifest(PATH_DB_ABSOLUTE, ${$self->{oTargetPath}}{$strTarget});
+            my $hTargetManifest = $oStorageDb->manifest(${$self->{oTargetPath}}{$strTarget});
 
             for my $strName (keys(%{$hTargetManifest}))
             {
@@ -578,7 +565,7 @@ sub clean
                 if (!$bDelta)
                 {
                     confess &log(ERROR, "cannot restore to path '${$self->{oTargetPath}}{$strTarget}' that contains files - " .
-                                        'try using --delta if this is what you intended', ERROR_RESTORE_PATH_NOT_EMPTY);
+                                        'try using --delta if this is what you intended', ERROR_PATH_NOT_EMPTY);
                 }
 
                 # Mark that some files were found
@@ -590,21 +577,21 @@ sub clean
     # Clean up each target starting from the most nested
     my %oFileChecked;
 
-    for my $strTarget (sort {$b cmp $a} ($oManifest->keys(MANIFEST_SECTION_BACKUP_TARGET)))
+    for my $strTarget ($oManifest->keys(MANIFEST_SECTION_BACKUP_TARGET, INI_SORT_REVERSE))
     {
         if ($oTargetFound{$strTarget})
         {
             &log(INFO, "remove invalid files/paths/links from ${$self->{oTargetPath}}{$strTarget}");
 
             # OK for the special tablespace path to not exist yet - it will be created later
-            if (!$self->{oFile}->exists(PATH_DB_ABSOLUTE, ${$self->{oTargetPath}}{$strTarget}) &&
+            if (!$oStorageDb->pathExists(${$self->{oTargetPath}}{$strTarget}) &&
                 $oManifest->isTargetTablespace($strTarget))
             {
                 next;
             }
 
             # Load path manifest so it can be compared to deleted files/paths/links that are not in the backup
-            my $hTargetManifest = $self->{oFile}->manifest(PATH_DB_ABSOLUTE, ${$self->{oTargetPath}}{$strTarget});
+            my $hTargetManifest = $oStorageDb->manifest(${$self->{oTargetPath}}{$strTarget});
 
             # If the target is a file it doesn't matter whether it already exists or not.
             if ($oManifest->isTargetFile($strTarget))
@@ -650,7 +637,7 @@ sub clean
                     {
                         &log(DETAIL, "set ownership ${strUser}:${strGroup} on ${strOsFile}");
 
-                        $self->{oFile}->owner(PATH_DB_ABSOLUTE, $strOsFile, $strUser, $strGroup);
+                        $oStorageDb->owner($strOsFile, $strUser, $strGroup);
                     }
 
                     # If a link does not have the same destination, then delete it (it will be recreated later)
@@ -660,7 +647,7 @@ sub clean
                             $hTargetManifest->{$strName}{link_destination})
                         {
                             &log(DETAIL, "remove link ${strOsFile} - destination changed");
-                            fileRemove($strOsFile);
+                            $oStorageDb->remove($strOsFile);
                         }
                     }
                     # Else if file/path mode does not match, fix it
@@ -702,10 +689,10 @@ sub clean
                         else
                         {
                             &log(DETAIL, "remove ${strType} ${strOsFile}");
-                            fileRemove($strOsFile);
+                            $oStorageDb->remove($strOsFile);
 
                             # Removing a file can be expensive so send protocol keep alive
-                            $self->{oProtocol}->keepAlive();
+                            protocolKeepAlive();
 
                             $oRemoveHash{$strSection} += 1;
                         }
@@ -778,6 +765,9 @@ sub build
             {name => 'oManifest'}
         );
 
+    # Db storage
+    my $oStorageDb = storageDb();
+
     # Create target paths (except for MANIFEST_TARGET_PGDATA because that must already exist)
     foreach my $strTarget ($oManifest->keys(MANIFEST_SECTION_BACKUP_TARGET))
     {
@@ -790,10 +780,10 @@ sub build
                 $strPath = dirname($strPath);
             }
 
-            if (!$self->{oFile}->exists(PATH_DB_ABSOLUTE, $strPath))
+            if (!$oStorageDb->pathExists($strPath))
             {
-                $self->{oFile}->pathCreate(PATH_DB_ABSOLUTE, $strPath,
-                    $oManifest->get(MANIFEST_SECTION_TARGET_PATH, $strTarget, MANIFEST_SUBKEY_MODE));
+                $oStorageDb->pathCreate(
+                    $strPath, {strMode => $oManifest->get(MANIFEST_SECTION_TARGET_PATH, $strTarget, MANIFEST_SUBKEY_MODE)});
 
                 # Set ownership (??? this could be done better inside the file functions)
                 my $strUser = $oManifest->get(MANIFEST_SECTION_TARGET_PATH, $strTarget, MANIFEST_SUBKEY_USER);
@@ -801,7 +791,7 @@ sub build
 
                 if ($strUser ne getpwuid($<) || $strGroup ne getgrgid($())
                 {
-                    $self->{oFile}->owner(PATH_DB_ABSOLUTE, $strPath, $strUser, $strGroup);
+                    $oStorageDb->owner($strPath, $strUser, $strGroup);
                 }
             }
         }
@@ -842,13 +832,13 @@ sub build
 
                     # If the path/link does not already exist then create it.  The clean() method should have determined if the
                     # permissions, destinations, etc. are correct
-                    if (!$self->{oFile}->exists(PATH_DB_ABSOLUTE, $strDbPath))
+                    if (!$oStorageDb->pathExists($strDbPath) && !$oStorageDb->exists($strDbPath))
                     {
                         # Create a path
                         if ($strSection eq &MANIFEST_SECTION_TARGET_PATH)
                         {
-                            $self->{oFile}->pathCreate(PATH_DB_ABSOLUTE, $strDbPath,
-                                $oManifest->get($strSection, $strName, MANIFEST_SUBKEY_MODE));
+                            $oStorageDb->pathCreate(
+                                $strDbPath, {strMode => $oManifest->get($strSection, $strName, MANIFEST_SUBKEY_MODE)});
                         }
                         # Else create a link
                         else
@@ -859,8 +849,10 @@ sub build
                             # In order to create relative links they must be converted to absolute links and then made relative
                             # again by linkCreate().  It's possible to modify linkCreate() to accept relative paths but that could
                             # have an impact elsewhere and doesn't seem worth it.
-                            $self->{oFile}->linkCreate(PATH_DB_ABSOLUTE, pathAbsolute(dirname($strDbPath), $strDestination),
-                                                       PATH_DB_ABSOLUTE, $strDbPath, undef, (index($strDestination, '/') != 0));
+                            $oStorageDb->linkCreate(
+                                $oStorageDb->pathAbsolute(
+                                    dirname($strDbPath), $strDestination), $strDbPath,
+                                    {bRelative => (index($strDestination, '/') != 0, bIgnoreExists => true)});
                         }
 
                         # Set ownership (??? this could be done better inside the file functions)
@@ -869,7 +861,7 @@ sub build
 
                         if ($strUser ne getpwuid($<) || $strGroup ne getgrgid($())
                         {
-                            $self->{oFile}->owner(PATH_DB_ABSOLUTE, $strDbPath, $strUser, $strGroup);
+                            $oStorageDb->owner($strDbPath, $strUser, $strGroup);
                         }
                     }
 
@@ -905,7 +897,7 @@ sub recovery
     my $strRecoveryConf = $self->{strDbClusterPath} . '/' . DB_FILE_RECOVERYCONF;
 
     # See if recovery.conf already exists
-    my $bRecoveryConfExists = $self->{oFile}->exists(PATH_DB_ABSOLUTE, $strRecoveryConf);
+    my $bRecoveryConfExists = storageDb()->exists($strRecoveryConf);
 
     # If RECOVERY_TYPE_PRESERVE then warn if recovery.conf does not exist and return
     if (optionTest(OPTION_TYPE, RECOVERY_TYPE_PRESERVE))
@@ -920,7 +912,7 @@ sub recovery
         # In all other cases the old recovery.conf should be removed if it exists
         if ($bRecoveryConfExists)
         {
-            $self->{oFile}->remove(PATH_DB_ABSOLUTE, $strRecoveryConf);
+            storageDb()->remove($strRecoveryConf);
         }
 
         # If RECOVERY_TYPE_NONE then return
@@ -1032,21 +1024,28 @@ sub process
     # Assign function parameters, defaults, and log debug info
     my ($strOperation) = logDebugParam (__PACKAGE__ . '->process');
 
-    if (!fileExists($self->{strDbClusterPath}))
+    # Db storage
+    my $oStorageDb = storageDb();
+
+    if (!$oStorageDb->pathExists($self->{strDbClusterPath}))
     {
         confess &log(ERROR, "\$PGDATA directory $self->{strDbClusterPath} does not exist");
     }
 
     # Make sure that Postgres is not running
-    if ($self->{oFile}->exists(PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . DB_FILE_POSTMASTERPID))
+    if ($oStorageDb->exists($self->{strDbClusterPath} . '/' . DB_FILE_POSTMASTERPID))
     {
-        confess &log(ERROR, 'unable to restore while Postgres is running', ERROR_POSTMASTER_RUNNING);
+        confess &log(ERROR,
+            "unable to restore while PostgreSQL is running\n" .
+            "HINT: presence of '" . DB_FILE_POSTMASTERPID . "' in '$self->{strDbClusterPath}' indicates PostgreSQL is running.\n" .
+            "HINT: remove '" . DB_FILE_POSTMASTERPID . "' only if PostgreSQL is not running.",
+            ERROR_POSTMASTER_RUNNING);
     }
 
     # If the restore will be destructive attempt to verify that $PGDATA is valid
     if ((optionGet(OPTION_DELTA) || optionGet(OPTION_FORCE)) &&
-        !($self->{oFile}->exists(PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . DB_FILE_PGVERSION) ||
-          $self->{oFile}->exists(PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . FILE_MANIFEST)))
+        !($oStorageDb->exists($self->{strDbClusterPath} . '/' . DB_FILE_PGVERSION) ||
+          $oStorageDb->exists($self->{strDbClusterPath} . '/' . FILE_MANIFEST)))
     {
         &log(WARN, '--delta or --force specified but unable to find \'' . DB_FILE_PGVERSION . '\' or \'' . FILE_MANIFEST .
                    '\' in \'' . $self->{strDbClusterPath} . '\' to confirm that this is a valid $PGDATA directory.' .
@@ -1058,12 +1057,13 @@ sub process
     }
 
     # Copy backup info, load it, then delete
-    $self->{oFile}->copy(PATH_BACKUP_CLUSTER, FILE_BACKUP_INFO,
-                         PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . FILE_BACKUP_INFO);
+    $oStorageDb->copy(
+        storageRepo()->openRead(STORAGE_REPO_BACKUP . qw(/) . FILE_BACKUP_INFO, {bProtocolCompress => true}),
+        $self->{strDbClusterPath} . '/' . FILE_BACKUP_INFO);
 
-    my $oBackupInfo = new pgBackRest::BackupInfo($self->{strDbClusterPath}, false);
+    my $oBackupInfo = new pgBackRest::Backup::Info($self->{strDbClusterPath}, false, undef, {oStorage => storageDb()});
 
-    $self->{oFile}->remove(PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . FILE_BACKUP_INFO, undef, false, true);
+    $oStorageDb->remove($self->{strDbClusterPath} . '/' . FILE_BACKUP_INFO);
 
     # If set to restore is latest then get the actual set
     if ($self->{strBackupSet} eq OPTION_DEFAULT_RESTORE_SET)
@@ -1092,9 +1092,10 @@ sub process
 
     # Delete pg_control file.  This will be copied from the backup at the very end to prevent a partially restored database
     # from being started by PostgreSQL.
-    $self->{oFile}->remove(PATH_DB_ABSOLUTE, $oManifest->dbPathGet(
-                           $oManifest->get(MANIFEST_SECTION_BACKUP_TARGET, MANIFEST_TARGET_PGDATA, MANIFEST_SUBKEY_PATH),
-                           MANIFEST_FILE_PGCONTROL), true, true);
+    $oStorageDb->remove(
+        $oManifest->dbPathGet(
+            $oManifest->get(MANIFEST_SECTION_BACKUP_TARGET, MANIFEST_TARGET_PGDATA, MANIFEST_SUBKEY_PATH),
+            MANIFEST_FILE_PGCONTROL));
 
     # Clean the restore paths
     $self->clean($oManifest);
@@ -1190,7 +1191,7 @@ sub process
     }
 
     # Initialize the restore process
-    my $oRestoreProcess = new pgBackRest::Protocol::LocalProcess(BACKUP);
+    my $oRestoreProcess = new pgBackRest::Protocol::Local::Process(BACKUP);
     $oRestoreProcess->hostAdd(1, optionGet(OPTION_PROCESS_MAX));
 
     # Variables used for parallel copy
@@ -1200,7 +1201,7 @@ sub process
     foreach my $strRepoFile (
         sort {sprintf("%016d-${b}", $oManifest->numericGet(MANIFEST_SECTION_TARGET_FILE, $b, MANIFEST_SUBKEY_SIZE)) cmp
               sprintf("%016d-${a}", $oManifest->numericGet(MANIFEST_SECTION_TARGET_FILE, $a, MANIFEST_SUBKEY_SIZE))}
-        ($oManifest->keys(MANIFEST_SECTION_TARGET_FILE, 'none')))
+        ($oManifest->keys(MANIFEST_SECTION_TARGET_FILE, INI_SORT_NONE)))
     {
         # Skip the tablespace_map file in versions >= 9.5 so Postgres does not rewrite links in DB_PATH_PGTBLSPC.
         # The tablespace links have already been created by Restore::build().
@@ -1226,7 +1227,7 @@ sub process
         # Copy pg_control to a temporary file that will be renamed later
         if ($strRepoFile eq MANIFEST_TARGET_PGDATA . '/' . DB_FILE_PGCONTROL)
         {
-            $strDbFile .= '.' . BACKREST_EXE;
+            $strDbFile .= '.' . STORAGE_TEMP_EXT;
         }
 
         # Increment file size
@@ -1260,26 +1261,32 @@ sub process
 
         # A keep-alive is required here because if there are a large number of resumed files that need to be checksummed
         # then the remote might timeout while waiting for a command.
-        $self->{oProtocol}->keepAlive();
+        protocolKeepAlive();
     }
 
     # Create recovery.conf file
     $self->recovery($oManifest->get(MANIFEST_SECTION_BACKUP_DB, MANIFEST_KEY_DB_VERSION));
 
     # Sync db cluster path
-    $self->{oFile}->pathSync(PATH_DB_ABSOLUTE, $self->{strDbClusterPath}, true);
+    $oStorageDb->pathSync($self->{strDbClusterPath}, {bRecurse => true});
 
-    # Copy pg_control last
+    # Move pg_control last
     &log(INFO,
         'restore ' . $oManifest->dbPathGet(undef, MANIFEST_FILE_PGCONTROL) .
-        ' (copied last to ensure aborted restores cannot be started)');
+        ' (performed last to ensure aborted restores cannot be started)');
 
-    $self->{oFile}->move(
-        PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . DB_FILE_PGCONTROL . '.' . BACKREST_EXE,
-        PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . DB_FILE_PGCONTROL, undef, true);
+    $oStorageDb->move(
+        $self->{strDbClusterPath} . '/' . DB_FILE_PGCONTROL . '.' . STORAGE_TEMP_EXT,
+        $self->{strDbClusterPath} . '/' . DB_FILE_PGCONTROL);
+
+    # Sync to be sure the rename of pg_control is persisted
+    $oStorageDb->pathSync($self->{strDbClusterPath});
 
     # Finally remove the manifest to indicate the restore is complete
-    $self->{oFile}->remove(PATH_DB_ABSOLUTE, $self->{strDbClusterPath} . '/' . FILE_MANIFEST, undef, false, true);
+    $oStorageDb->remove($self->{strDbClusterPath} . '/' . FILE_MANIFEST);
+
+    # Sync removal of the manifest
+    $oStorageDb->pathSync($self->{strDbClusterPath});
 
     # Return from function and log return values if any
     return logDebugReturn($strOperation);
